package config

import (
	"fmt"
	"io"
	"path"
	"reflect"
	"strings"

	"github.com/moshebe/gebug/pkg/render"
	"github.com/spf13/afero"
	"gopkg.in/yaml.v2"
)

// AppFs hold the file-system abstraction for this package
var AppFs = afero.NewOsFs()

// Config contains the fields of gebug configuration
type Config struct {
	Name             string   `yaml:"name" json:"name"`
	OutputBinaryPath string   `yaml:"output_binary" json:"outputBinary"`
	PreRunCommands   []string `yaml:"pre_run_commands,omitempty" json:"preRunCommands"`
	BuildCommand     string   `yaml:"build_command" json:"buildCommand"`
	BuildDir         string   `yaml:"build_dir" json:"buildDir"`
	RunCommand       string   `yaml:"run_command" json:"runCommand"`
	RuntimeImage     string   `yaml:"runtime_image" json:"runtimeImage"`
	DebuggerEnabled  bool     `yaml:"debugger_enabled" json:"debuggerEnabled"`
	DebuggerPort     int      `yaml:"debugger_port" json:"debuggerPort"`
	ExposePorts      []string `yaml:"expose_ports" json:"exposePorts"`
	Networks         []string `yaml:"networks" json:"networks"`
	Environment      []string `yaml:"environment" json:"environment"`
}

func updateBuildCommand(buildCommand string, debuggerEnabled bool) string {
	buildCommand = strings.TrimSpace(buildCommand)
	if !debuggerEnabled || strings.Contains(buildCommand, "gcflags") {
		return buildCommand
	}

	goBuildPrefix := "go build"
	commandArgs := strings.TrimPrefix(buildCommand, goBuildPrefix)
	return strings.TrimSpace(goBuildPrefix + ` -gcflags="all=-N -l"` + commandArgs)
}

// Load loads a configuration to a Config struct
func Load(input []byte) (*Config, error) {
	c := &Config{}
	err := yaml.Unmarshal(input, c)
	if err != nil {
		return nil, fmt.Errorf("unmarshal configuration: %w", err)
	}

	err = c.render()
	if err != nil {
		return nil, fmt.Errorf("render configuration: %w", err)
	}

	c.BuildCommand = updateBuildCommand(c.BuildCommand, c.DebuggerEnabled)
	return c, nil
}

// ResolvePath of the project while taking care valid suffixes like '/', '.gebug' etc.
func ResolvePath(projectPath string) string {
	if strings.HasSuffix(projectPath, Path) {
		return projectPath
	}

	if strings.HasSuffix(projectPath, RootDir) || strings.HasSuffix(projectPath, RootDir+"/") {
		return path.Join(projectPath, Path)
	}

	return FilePath(projectPath, Path)
}

// Write the serialized configuration
func (c Config) Write(writer io.Writer) error {
	out, err := yaml.Marshal(c)
	if err != nil {
		return fmt.Errorf("marshal configuration: %w", err)
	}
	_, err = writer.Write(out)
	if err != nil {
		return fmt.Errorf("write marshalled configuration: %w", err)
	}
	return nil
}

func (c *Config) render() error {
	values := map[string]string{}

	typeElement := reflect.TypeOf(c).Elem()
	valueElement := reflect.ValueOf(c).Elem()
	fields := typeElement.NumField()

	for i := 0; i < fields; i++ {
		tagName := typeElement.Field(i).Tag.Get("yaml")
		values[tagName] = valueElement.Field(i).String()
	}

	for i := 0; i < fields; i++ {
		fieldType := typeElement.Field(i)
		if fieldType.Type.String() != "string" {
			continue
		}

		rawValue := valueElement.Field(i).String()
		newValue, err := render.Render(rawValue, values)
		if err != nil {
			return fmt.Errorf("unable to render value %q: %w", rawValue, err)
		}
		valueElement.Field(i).SetString(newValue)
	}
	return nil
}
